@@ -1,4 +1,4 @@
-from typing import Iterator, List, Optional
+from typing import Iterator, Optional
 
 from langchain_core.documents import Document
 
@@ -32,9 +32,6 @@ class PubMedLoader(BaseLoader):
             top_k_results=load_max_docs,
         )
 
-    def load(self) -> List[Document]:
-        return list(self._client.lazy_load_docs(self.query))
-
     def lazy_load(self) -> Iterator[Document]:
         for doc in self._client.lazy_load_docs(self.query):
             yield doc
