@@ -35,7 +35,7 @@ def load_llm(file: Union[str, Path]) -> BaseLLM:
     if file_path.suffix == ".json":
         with open(file_path) as f:
             config = json.load(f)
-    elif file_path.suffix == ".yaml":
+    elif file_path.suffix.endswith((".yaml", ".yml")):
         with open(file_path, "r") as f:
             config = yaml.safe_load(f)
     else:
