@@ -3,6 +3,7 @@ from __future__ import annotations
 from concurrent.futures import Executor, ThreadPoolExecutor
 from typing import TYPE_CHECKING, Any, ClassVar, Dict, Iterator, List, Optional, Union
 
+from langchain_core._api.deprecation import deprecated
 from langchain_core.callbacks.manager import (
     AsyncCallbackManagerForLLMRun,
     CallbackManagerForLLMRun,
@@ -48,7 +49,7 @@ def is_gemini_model(model_name: str) -> bool:
     return model_name is not None and "gemini" in model_name
 
 
-def completion_with_retry(
+def completion_with_retry(  # type: ignore[no-redef]
     llm: VertexAI,
     prompt: List[Union[str, "Image"]],
     stream: bool = False,
@@ -200,6 +201,11 @@ class _VertexAICommon(_VertexAIBase):
         return params
 
 
+@deprecated(
+    since="0.0.12",
+    removal="0.2.0",
+    alternative_import="langchain_google_vertexai.VertexAI",
+)
 class VertexAI(_VertexAICommon, BaseLLM):
     """Google Vertex AI large language models."""
 
@@ -324,7 +330,7 @@ class VertexAI(_VertexAICommon, BaseLLM):
                     generation += chunk
                 generations.append([generation])
             else:
-                res = completion_with_retry(
+                res = completion_with_retry(  # type: ignore[misc]
                     self,
                     [prompt],
                     stream=should_stream,
@@ -367,7 +373,7 @@ class VertexAI(_VertexAICommon, BaseLLM):
         **kwargs: Any,
     ) -> Iterator[GenerationChunk]:
         params = self._prepare_params(stop=stop, stream=True, **kwargs)
-        for stream_resp in completion_with_retry(
+        for stream_resp in completion_with_retry(  # type: ignore[misc]
             self,
             [prompt],
             stream=True,
@@ -376,15 +382,20 @@ class VertexAI(_VertexAICommon, BaseLLM):
             **params,
         ):
             chunk = self._response_to_generation(stream_resp)
-            yield chunk
             if run_manager:
                 run_manager.on_llm_new_token(
                     chunk.text,
                     chunk=chunk,
                     verbose=self.verbose,
                 )
+            yield chunk
 
 
+@deprecated(
+    since="0.0.12",
+    removal="0.2.0",
+    alternative_import="langchain_google_vertexai.VertexAIModelGarden",
+)
 class VertexAIModelGarden(_VertexAIBase, BaseLLM):
     """Large language models served from Vertex AI Model Garden."""
 
