@@ -118,6 +118,38 @@ def _import_brave_search_tool() -> Any:
     return BraveSearch
 
 
+def _import_cogniswitch_store_file_tool() -> Any:
+    from langchain_community.tools.cogniswitch.tool import (
+        CogniswitchKnowledgeSourceFile,
+    )
+
+    return CogniswitchKnowledgeSourceFile
+
+
+def _import_cogniswitch_store_url_tool() -> Any:
+    from langchain_community.tools.cogniswitch.tool import CogniswitchKnowledgeSourceURL
+
+    return CogniswitchKnowledgeSourceURL
+
+
+def _import_cogniswitch_answer_tool() -> Any:
+    from langchain_community.tools.cogniswitch.tool import CogniswitchKnowledgeRequest
+
+    return CogniswitchKnowledgeRequest
+
+
+def _import_cogniswitch_knowledge_status_tool() -> Any:
+    from langchain_community.tools.cogniswitch.tool import CogniswitchKnowledgeStatus
+
+    return CogniswitchKnowledgeStatus
+
+
+def _import_connery_tool() -> Any:
+    from langchain_community.tools.connery import ConneryAction
+
+    return ConneryAction
+
+
 def _import_ddg_search_tool_DuckDuckGoSearchResults() -> Any:
     from langchain_community.tools.ddg_search.tool import DuckDuckGoSearchResults
 
@@ -472,6 +504,24 @@ def _import_plugin() -> Any:
     return AIPluginTool
 
 
+def _import_polygon_tool_PolygonFinancials() -> Any:
+    from langchain_community.tools.polygon.financials import PolygonFinancials
+
+    return PolygonFinancials
+
+
+def _import_polygon_tool_PolygonLastQuote() -> Any:
+    from langchain_community.tools.polygon.last_quote import PolygonLastQuote
+
+    return PolygonLastQuote
+
+
+def _import_polygon_tool_PolygonTickerNews() -> Any:
+    from langchain_community.tools.polygon.ticker_news import PolygonTickerNews
+
+    return PolygonTickerNews
+
+
 def _import_powerbi_tool_InfoPowerBITool() -> Any:
     from langchain_community.tools.powerbi.tool import InfoPowerBITool
 
@@ -525,7 +575,9 @@ def _import_reddit_search_RedditSearchRun() -> Any:
 
 
 def _import_render() -> Any:
-    from langchain_community.tools.render import format_tool_to_openai_function
+    from langchain_community.tools.convert_to_openai import (
+        format_tool_to_openai_function,
+    )
 
     return format_tool_to_openai_function
 
@@ -730,6 +782,12 @@ def _import_yahoo_finance_news() -> Any:
     return YahooFinanceNewsTool
 
 
+def _import_you_tool() -> Any:
+    from langchain_community.tools.you.tool import YouSearchTool
+
+    return YouSearchTool
+
+
 def _import_youtube_search() -> Any:
     from langchain_community.tools.youtube.search import YouTubeSearchTool
 
@@ -789,6 +847,16 @@ def __getattr__(name: str) -> Any:
         return _import_bing_search_tool_BingSearchRun()
     elif name == "BraveSearch":
         return _import_brave_search_tool()
+    elif name == "CogniswitchKnowledgeSourceFile":
+        return _import_cogniswitch_store_file_tool()
+    elif name == "CogniswitchKnowledgeSourceURL":
+        return _import_cogniswitch_store_url_tool()
+    elif name == "CogniswitchKnowledgeRequest":
+        return _import_cogniswitch_answer_tool()
+    elif name == "CogniswitchKnowledgeStatus":
+        return _import_cogniswitch_knowledge_status_tool()
+    elif name == "ConneryAction":
+        return _import_connery_tool()
     elif name == "DuckDuckGoSearchResults":
         return _import_ddg_search_tool_DuckDuckGoSearchResults()
     elif name == "DuckDuckGoSearchRun":
@@ -905,6 +973,12 @@ def __getattr__(name: str) -> Any:
         return _import_playwright_NavigateTool()
     elif name == "AIPluginTool":
         return _import_plugin()
+    elif name == "PolygonFinancials":
+        return _import_polygon_tool_PolygonFinancials()
+    elif name == "PolygonLastQuote":
+        return _import_polygon_tool_PolygonLastQuote()
+    elif name == "PolygonTickerNews":
+        return _import_polygon_tool_PolygonTickerNews()
     elif name == "InfoPowerBITool":
         return _import_powerbi_tool_InfoPowerBITool()
     elif name == "ListPowerBITool":
@@ -987,6 +1061,8 @@ def __getattr__(name: str) -> Any:
         return _import_wolfram_alpha_tool()
     elif name == "YahooFinanceNewsTool":
         return _import_yahoo_finance_news()
+    elif name == "YouSearchTool":
+        return _import_you_tool()
     elif name == "YouTubeSearchTool":
         return _import_youtube_search()
     elif name == "ZapierNLAListActions":
@@ -1025,6 +1101,11 @@ __all__ = [
     "BingSearchRun",
     "BraveSearch",
     "ClickTool",
+    "CogniswitchKnowledgeSourceFile",
+    "CogniswitchKnowledgeSourceURL",
+    "CogniswitchKnowledgeRequest",
+    "CogniswitchKnowledgeStatus",
+    "ConneryAction",
     "CopyFileTool",
     "CurrentWebPageTool",
     "DeleteFileTool",
@@ -1083,6 +1164,9 @@ __all__ = [
     "OpenAPISpec",
     "OpenWeatherMapQueryRun",
     "PubmedQueryRun",
+    "PolygonFinancials",
+    "PolygonLastQuote",
+    "PolygonTickerNews",
     "RedditSearchRun",
     "QueryCheckerTool",
     "QueryPowerBITool",
@@ -1116,6 +1200,7 @@ __all__ = [
     "WolframAlphaQueryRun",
     "WriteFileTool",
     "YahooFinanceNewsTool",
+    "YouSearchTool",
     "YouTubeSearchTool",
     "ZapierNLAListActions",
     "ZapierNLARunAction",
