@@ -11,7 +11,7 @@ from langchain_community.utilities.golden_query import GoldenQueryAPIWrapper
 class GoldenQueryRun(BaseTool):
     """Tool that adds the capability to query using the Golden API and get back JSON."""
 
-    name: str = "Golden-Query"
+    name: str = "golden_query"
     description: str = (
         "A wrapper around Golden Query API."
         " Useful for getting entities that match"
