@@ -74,6 +74,12 @@ def _import_annoy() -> Any:
     return Annoy
 
 
+def _import_apache_doris() -> Any:
+    from langchain_community.vectorstores.apache_doris import ApacheDoris
+
+    return ApacheDoris
+
+
 def _import_atlas() -> Any:
     from langchain_community.vectorstores.atlas import AtlasDB
 
@@ -104,6 +110,14 @@ def _import_baiducloud_vector_search() -> Any:
     return BESVectorStore
 
 
+def _import_bigquery() -> Any:
+    from langchain_community.vectorstores.bigquery_vector_search import (
+        BigQueryVectorSearch,
+    )
+
+    return BigQueryVectorSearch
+
+
 def _import_cassandra() -> Any:
     from langchain_community.vectorstores.cassandra import Cassandra
 
@@ -178,6 +192,14 @@ def _import_docarray_inmemory() -> Any:
     return DocArrayInMemorySearch
 
 
+def _import_documentdb() -> Any:
+    from langchain_community.vectorstores.documentdb import (
+        DocumentDBVectorSearch,
+    )
+
+    return DocumentDBVectorSearch
+
+
 def _import_elasticsearch() -> Any:
     from langchain_community.vectorstores.elasticsearch import ElasticsearchStore
 
@@ -196,12 +218,48 @@ def _import_faiss() -> Any:
     return FAISS
 
 
+def _import_hanavector() -> Any:
+    from langchain_community.vectorstores.hanavector import HanaDB
+
+    return HanaDB
+
+
+def _import_kinetica() -> Any:
+    from langchain_community.vectorstores.kinetica import Kinetica
+
+    return Kinetica
+
+
+def _import_kinetica_settings() -> Any:
+    from langchain_community.vectorstores.kinetica import KineticaSettings
+
+    return KineticaSettings
+
+
+def _import_distance_strategy() -> Any:
+    from langchain_community.vectorstores.kinetica import DistanceStrategy
+
+    return DistanceStrategy
+
+
 def _import_hologres() -> Any:
     from langchain_community.vectorstores.hologres import Hologres
 
     return Hologres
 
 
+def _import_infinispanvs() -> Any:
+    from langchain_community.vectorstores.infinispanvs import InfinispanVS
+
+    return InfinispanVS
+
+
+def _import_kdbai() -> Any:
+    from langchain_community.vectorstores.kdbai import KDBAI
+
+    return KDBAI
+
+
 def _import_lancedb() -> Any:
     from langchain_community.vectorstores.lancedb import LanceDB
 
@@ -360,6 +418,12 @@ def _import_supabase() -> Any:
     return SupabaseVectorStore
 
 
+def _import_surrealdb() -> Any:
+    from langchain_community.vectorstores.surrealdb import SurrealDBStore
+
+    return SurrealDBStore
+
+
 def _import_tair() -> Any:
     from langchain_community.vectorstores.tair import Tair
 
@@ -444,6 +508,18 @@ def _import_zilliz() -> Any:
     return Zilliz
 
 
+def _import_neuraldb() -> Any:
+    from langchain_community.vectorstores.thirdai_neuraldb import NeuralDBVectorStore
+
+    return NeuralDBVectorStore
+
+
+def _import_lantern() -> Any:
+    from langchain_community.vectorstores.lantern import Lantern
+
+    return Lantern
+
+
 def __getattr__(name: str) -> Any:
     if name == "AnalyticDB":
         return _import_analyticdb()
@@ -459,6 +535,8 @@ def __getattr__(name: str) -> Any:
         return _import_elastic_vector_search()
     elif name == "Annoy":
         return _import_annoy()
+    elif name == "ApacheDoris":
+        return _import_apache_doris()
     elif name == "AtlasDB":
         return _import_atlas()
     elif name == "AwaDB":
@@ -467,6 +545,8 @@ def __getattr__(name: str) -> Any:
         return _import_azuresearch()
     elif name == "Bagel":
         return _import_bageldb()
+    elif name == "BigQueryVectorSearch":
+        return _import_bigquery()
     elif name == "BESVectorStore":
         return _import_baiducloud_vector_search()
     elif name == "Cassandra":
@@ -491,6 +571,8 @@ def __getattr__(name: str) -> Any:
         return _import_dingo()
     elif name == "DocArrayInMemorySearch":
         return _import_docarray_inmemory()
+    elif name == "DocumentDBVectorSearch":
+        return _import_documentdb()
     elif name == "DocArrayHnswSearch":
         return _import_docarray_hnsw()
     elif name == "ElasticsearchStore":
@@ -499,8 +581,20 @@ def __getattr__(name: str) -> Any:
         return _import_epsilla()
     elif name == "FAISS":
         return _import_faiss()
+    elif name == "HanaDB":
+        return _import_hanavector()
     elif name == "Hologres":
         return _import_hologres()
+    elif name == "InfinispanVS":
+        return _import_infinispanvs()
+    elif name == "KDBAI":
+        return _import_kdbai()
+    elif name == "DistanceStrategy":
+        return _import_distance_strategy()
+    elif name == "KineticaSettings":
+        return _import_kinetica_settings()
+    elif name == "Kinetica":
+        return _import_kinetica()
     elif name == "LanceDB":
         return _import_lancedb()
     elif name == "LLMRails":
@@ -551,6 +645,8 @@ def __getattr__(name: str) -> Any:
         return _import_starrocks()
     elif name == "SupabaseVectorStore":
         return _import_supabase()
+    elif name == "SurrealDBStore":
+        return _import_surrealdb()
     elif name == "Tair":
         return _import_tair()
     elif name == "TencentVectorDB":
@@ -581,6 +677,10 @@ def __getattr__(name: str) -> Any:
         return _import_zilliz()
     elif name == "VespaStore":
         return _import_vespa()
+    elif name == "NeuralDBVectorStore":
+        return _import_neuraldb()
+    elif name == "Lantern":
+        return _import_lantern()
     else:
         raise AttributeError(f"Could not find: {name}")
 
@@ -590,6 +690,7 @@ __all__ = [
     "AlibabaCloudOpenSearchSettings",
     "AnalyticDB",
     "Annoy",
+    "ApacheDoris",
     "AtlasDB",
     "AwaDB",
     "AzureSearch",
@@ -611,7 +712,13 @@ __all__ = [
     "ElasticsearchStore",
     "Epsilla",
     "FAISS",
+    "HanaDB",
     "Hologres",
+    "InfinispanVS",
+    "KDBAI",
+    "DistanceStrategy",
+    "Kinetica",
+    "KineticaSettings",
     "LanceDB",
     "LLMRails",
     "Marqo",
@@ -637,6 +744,7 @@ __all__ = [
     "SQLiteVSS",
     "StarRocks",
     "SupabaseVectorStore",
+    "SurrealDBStore",
     "Tair",
     "TileDB",
     "Tigris",
@@ -654,4 +762,7 @@ __all__ = [
     "TencentVectorDB",
     "AzureCosmosDBVectorSearch",
     "VectorStore",
+    "NeuralDBVectorStore",
+    "Lantern",
+    "DocumentDBVectorSearch",
 ]
