@@ -209,7 +209,7 @@ class Chroma(VectorStore):
                     empty_ids.append(idx)
             if non_empty_ids:
                 metadatas = [metadatas[idx] for idx in non_empty_ids]
-                images_with_metadatas = [uris[idx] for idx in non_empty_ids]
+                images_with_metadatas = [b64_texts[idx] for idx in non_empty_ids]
                 embeddings_with_metadatas = (
                     [embeddings[idx] for idx in non_empty_ids] if embeddings else None
                 )
@@ -225,13 +225,13 @@ class Chroma(VectorStore):
                     if "Expected metadata value to be" in str(e):
                         msg = (
                             "Try filtering complex metadata using "
-                            "langchain.vectorstores.utils.filter_complex_metadata."
+                            "langchain_community.vectorstores.utils.filter_complex_metadata."
                         )
                         raise ValueError(e.args[0] + "\n\n" + msg)
                     else:
                         raise e
             if empty_ids:
-                images_without_metadatas = [uris[j] for j in empty_ids]
+                images_without_metadatas = [b64_texts[j] for j in empty_ids]
                 embeddings_without_metadatas = (
                     [embeddings[j] for j in empty_ids] if embeddings else None
                 )
@@ -304,7 +304,7 @@ class Chroma(VectorStore):
                     if "Expected metadata value to be" in str(e):
                         msg = (
                             "Try filtering complex metadata from the document using "
-                            "langchain.vectorstores.utils.filter_complex_metadata."
+                            "langchain_community.vectorstores.utils.filter_complex_metadata."
                         )
                         raise ValueError(e.args[0] + "\n\n" + msg)
                     else:
@@ -345,7 +345,9 @@ class Chroma(VectorStore):
         Returns:
             List[Document]: List of documents most similar to the query text.
         """
-        docs_and_scores = self.similarity_search_with_score(query, k, filter=filter)
+        docs_and_scores = self.similarity_search_with_score(
+            query, k, filter=filter, **kwargs
+        )
         return [doc for doc, _ in docs_and_scores]
 
     def similarity_search_by_vector(
@@ -369,6 +371,7 @@ class Chroma(VectorStore):
             n_results=k,
             where=filter,
             where_document=where_document,
+            **kwargs,
         )
         return _results_to_docs(results)
 
@@ -398,6 +401,7 @@ class Chroma(VectorStore):
             n_results=k,
             where=filter,
             where_document=where_document,
+            **kwargs,
         )
         return _results_to_docs_and_scores(results)
 
@@ -427,6 +431,7 @@ class Chroma(VectorStore):
                 n_results=k,
                 where=filter,
                 where_document=where_document,
+                **kwargs,
             )
         else:
             query_embedding = self._embedding_function.embed_query(query)
@@ -435,6 +440,7 @@ class Chroma(VectorStore):
                 n_results=k,
                 where=filter,
                 where_document=where_document,
+                **kwargs,
             )
 
         return _results_to_docs_and_scores(results)
@@ -505,6 +511,7 @@ class Chroma(VectorStore):
             where=filter,
             where_document=where_document,
             include=["metadatas", "documents", "distances", "embeddings"],
+            **kwargs,
         )
         mmr_selected = maximal_marginal_relevance(
             np.array(embedding, dtype=np.float32),
