@@ -9,6 +9,7 @@ try:
     import deeplake
     from deeplake import VectorStore as DeepLakeVectorStore
     from deeplake.core.fast_forwarding import version_compare
+    from deeplake.util.exceptions import SampleExtendError
 
     _DEEPLAKE_INSTALLED = True
 except ImportError:
@@ -51,6 +52,7 @@ class DeepLake(VectorStore):
     """
 
     _LANGCHAIN_DEFAULT_DEEPLAKE_PATH = "./deeplake/"
+    _valid_search_kwargs = ["lambda_mult"]
 
     def __init__(
         self,
@@ -59,7 +61,7 @@ class DeepLake(VectorStore):
         embedding: Optional[Embeddings] = None,
         embedding_function: Optional[Embeddings] = None,
         read_only: bool = False,
-        ingestion_batch_size: int = 1000,
+        ingestion_batch_size: int = 1024,
         num_workers: int = 0,
         verbose: bool = True,
         exec_option: Optional[str] = None,
@@ -84,8 +86,19 @@ class DeepLake(VectorStore):
             ... )
 
         Args:
-            dataset_path (str): Path to existing dataset or where to create
-                a new one. Defaults to _LANGCHAIN_DEFAULT_DEEPLAKE_PATH.
+            dataset_path (str): The full path for storing to the Deep Lake
+                Vector Store. It can be:
+                - a Deep Lake cloud path of the form ``hub://org_id/dataset_name``.
+                    Requires registration with Deep Lake.
+                - an s3 path of the form ``s3://bucketname/path/to/dataset``.
+                    Credentials are required in either the environment or passed to
+                    the creds argument.
+                - a local file system path of the form ``./path/to/dataset``
+                    or ``~/path/to/dataset`` or ``path/to/dataset``.
+                - a memory path of the form ``mem://path/to/dataset`` which doesn't
+                    save the dataset but keeps it in memory instead.
+                    Should be used only for testing as it does not persist.
+                    Defaults to _LANGCHAIN_DEFAULT_DEEPLAKE_PATH.
             token (str, optional):  Activeloop token, for fetching credentials
                 to the dataset at path if it is a Deep Lake dataset.
                 Tokens are normally autogenerated. Optional.
@@ -97,25 +110,29 @@ class DeepLake(VectorStore):
             read_only (bool): Open dataset in read-only mode. Default is False.
             ingestion_batch_size (int): During data ingestion, data is divided
                 into batches. Batch size is the size of each batch.
-                Default is 1000.
+                Default is 1024.
             num_workers (int): Number of workers to use during data ingestion.
                 Default is 0.
             verbose (bool): Print dataset summary after each operation.
                 Default is True.
-            exec_option (str, optional): DeepLakeVectorStore supports 3 ways to perform
-                searching - "python", "compute_engine", "tensor_db" and auto.
-                Default is None.
+            exec_option (str, optional): Default method for search execution.
+                It could be either ``"auto"``, ``"python"``, ``"compute_engine"``
+                or ``"tensor_db"``. Defaults to ``"auto"``.
+                If None, it's set to "auto".
                 - ``auto``- Selects the best execution method based on the storage
                     location of the Vector Store. It is the default option.
-                - ``python`` - Pure-python implementation that runs on the client.
-                    WARNING: using this with big datasets can lead to memory
-                    issues. Data can be stored anywhere.
-                - ``compute_engine`` - C++ implementation of the Deep Lake Compute
-                    Engine that runs on the client. Can be used for any data stored in
-                    or connected to Deep Lake. Not for in-memory or local datasets.
-                - ``tensor_db`` - Hosted Managed Tensor Database that is
-                    responsible for storage and query execution. Only for data stored in
-                    the Deep Lake Managed Database. Use runtime = {"db_engine": True}
+                - ``python`` - Pure-python implementation that runs on the client and
+                    can be used for data stored anywhere. WARNING: using this option
+                    with big datasets is discouraged because it can lead to
+                    memory issues.
+                - ``compute_engine`` - Performant C++ implementation of the Deep Lake
+                    Compute Engine that runs on the client and can be used for any data
+                    stored in or connected to Deep Lake. It cannot be used with
+                    in-memory or local datasets.
+                - ``tensor_db`` - Performant and fully-hosted Managed Tensor Database
+                    that is responsible for storage and query execution. Only available
+                    for data stored in the Deep Lake Managed Database. Store datasets
+                    in this database by specifying runtime = {"tensor_db": True}
                     during dataset creation.
             runtime (Dict, optional): Parameters for creating the Vector Store in
                 Deep Lake's Managed Tensor Database. Not applicable when loading an
@@ -219,11 +236,7 @@ class DeepLake(VectorStore):
         Returns:
             List[str]: List of IDs of the added texts.
         """
-        if kwargs:
-            unsupported_items = "`, `".join(set(kwargs.keys()))
-            raise TypeError(
-                f"`{unsupported_items}` is/are not a valid argument to add_text method"
-            )
+        self._validate_kwargs(kwargs, "add_texts")
 
         kwargs = {}
         if ids:
@@ -243,15 +256,25 @@ class DeepLake(VectorStore):
         elif len(texts) == 0:
             raise ValueError("`texts` parameter shouldn't be empty.")
 
-        return self.vectorstore.add(
-            text=texts,
-            metadata=metadatas,
-            embedding_data=texts,
-            embedding_tensor="embedding",
-            embedding_function=self._embedding_function.embed_documents,  # type: ignore
-            return_ids=True,
-            **kwargs,
-        )
+        try:
+            return self.vectorstore.add(
+                text=texts,
+                metadata=metadatas,
+                embedding_data=texts,
+                embedding_tensor="embedding",
+                embedding_function=self._embedding_function.embed_documents,  # type: ignore
+                return_ids=True,
+                **kwargs,
+            )
+        except SampleExtendError as e:
+            if "Failed to append a sample to the tensor 'metadata'" in str(e):
+                msg = (
+                    "**Hint: You might be using invalid type of argument in "
+                    "document loader (e.g. 'pathlib.PosixPath' instead of 'str')"
+                )
+                raise ValueError(e.args[0] + "\n\n" + msg)
+            else:
+                raise e
 
     def _search_tql(
         self,
@@ -371,6 +394,9 @@ class DeepLake(VectorStore):
         Raises:
             ValueError: if both `embedding` and `embedding_function` are not specified.
         """
+        if kwargs.get("tql_query"):
+            logger.warning("`tql_query` is deprecated. Please use `tql` instead.")
+            kwargs["tql"] = kwargs.pop("tql_query")
 
         if kwargs.get("tql"):
             return self._search_tql(
@@ -384,6 +410,8 @@ class DeepLake(VectorStore):
                 filter=filter,
             )
 
+        self._validate_kwargs(kwargs, "search")
+
         if embedding_function:
             if isinstance(embedding_function, Embeddings):
                 _embedding_function = embedding_function.embed_query
@@ -417,7 +445,6 @@ class DeepLake(VectorStore):
             return_tensors=["embedding", "metadata", "text", self._id_tensor_name],
             deep_memory=deep_memory,
         )
-
         scores = result["score"]
         embeddings = result["embedding"]
         metadatas = result["metadata"]
@@ -445,6 +472,9 @@ class DeepLake(VectorStore):
         ]
 
         if return_score:
+            if not isinstance(scores, list):
+                scores = [scores]
+
             return [(doc, score) for doc, score in zip(docs, scores)]
 
         return docs
@@ -899,3 +929,30 @@ class DeepLake(VectorStore):
             "better to use `db.vectorstore.dataset` instead."
         )
         return self.vectorstore.dataset
+
+    @classmethod
+    def _validate_kwargs(cls, kwargs, method_name):  # type: ignore[no-untyped-def]
+        if kwargs:
+            valid_items = cls._get_valid_args(method_name)
+            unsupported_items = cls._get_unsupported_items(kwargs, valid_items)
+
+            if unsupported_items:
+                raise TypeError(
+                    f"`{unsupported_items}` are not a valid "
+                    f"argument to {method_name} method"
+                )
+
+    @classmethod
+    def _get_valid_args(cls, method_name):  # type: ignore[no-untyped-def]
+        if method_name == "search":
+            return cls._valid_search_kwargs
+        else:
+            return []
+
+    @staticmethod
+    def _get_unsupported_items(kwargs, valid_items):  # type: ignore[no-untyped-def]
+        kwargs = {k: v for k, v in kwargs.items() if k not in valid_items}
+        unsupported_items = None
+        if kwargs:
+            unsupported_items = "`, `".join(set(kwargs.keys()))
+        return unsupported_items
