@@ -37,23 +37,34 @@ def dependable_tiledb_import() -> Any:
 
 
 def get_vector_index_uri_from_group(group: Any) -> str:
+    """Get the URI of the vector index."""
     return group[VECTOR_INDEX_NAME].uri
 
 
 def get_documents_array_uri_from_group(group: Any) -> str:
+    """Get the URI of the documents array from group.
+
+    Args:
+        group: TileDB group object.
+
+    Returns:
+        URI of the documents array.
+    """
     return group[DOCUMENTS_ARRAY_NAME].uri
 
 
 def get_vector_index_uri(uri: str) -> str:
+    """Get the URI of the vector index."""
     return f"{uri}/{VECTOR_INDEX_NAME}"
 
 
 def get_documents_array_uri(uri: str) -> str:
+    """Get the URI of the documents array."""
     return f"{uri}/{DOCUMENTS_ARRAY_NAME}"
 
 
 class TileDB(VectorStore):
-    """Wrapper around TileDB vector database.
+    """TileDB vector store.
 
     To use, you should have the ``tiledb-vector-search`` python package installed.
 
@@ -76,9 +87,28 @@ class TileDB(VectorStore):
         docs_array_uri: str = "",
         config: Optional[Mapping[str, Any]] = None,
         timestamp: Any = None,
+        allow_dangerous_deserialization: bool = False,
         **kwargs: Any,
     ):
-        """Initialize with necessary components."""
+        """Initialize with necessary components.
+
+        Args:
+            allow_dangerous_deserialization: whether to allow deserialization
+                of the data which involves loading data using pickle.
+                data can be modified by malicious actors to deliver a
+                malicious payload that results in execution of
+                arbitrary code on your machine.
+        """
+        if not allow_dangerous_deserialization:
+            raise ValueError(
+                "TileDB relies on pickle for serialization and deserialization. "
+                "This can be dangerous if the data is intercepted and/or modified "
+                "by malicious actors prior to being de-serialized. "
+                "If you are sure that the data is safe from modification, you can "
+                " set allow_dangerous_deserialization=True to proceed. "
+                "Loading of compromised data using pickle can result in execution of "
+                "arbitrary code on your machine."
+            )
         self.embedding = embedding
         self.embedding_function = embedding.embed_query
         self.index_uri = index_uri
